/* ========================================================================== */
/*	     Xtest18_5.c : User-defined printf() Function for DBGU	      */
/* ========================================================================== */
/*			  Designed and programmed by Duck-Yong Yoon in 2007.  */

#include <stdio.h>
#include "AT91SAM7S256.h"
#include "lib_AT91SAM7S256.h"
#include "OK7S256ads.h"

void DBGU_TX_char(unsigned int data)		/* transmit a character by DBGU */
{
  while(!(*AT91C_DBGU_CSR & 0x0002));		// wait until TXRDY=1

  *AT91C_DBGU_THR = data;			// transmit a character
}

int fputc(int ch, FILE *f)			// for the user-defined printf()
{
  if(ch == '\n')
    { DBGU_TX_char('\r');			// (CR character = 0x0D)
      DBGU_TX_char('\n');			// (LF character = 0x0A)
    }
  else
    DBGU_TX_char(ch);

  return ch;
}

int main(void)
{ unsigned int i;
  double x;

  MCU_initialize();				// initialize AT91SAM7S256 & kit
  Delay_ms(50);					// wait for system stabilization
  LCD_initialize();				// initialize text LCD
  Beep();

  LCD_string(0x80,"    printf()    ");		// display title
  LCD_string(0xC0,"    for DBGU    ");

  AT91F_DBGU_CfgPMC();				// enable clock of DBGU
  AT91F_DBGU_CfgPIO();				// DTXD, DRXD by Peripheral A

  *AT91C_DBGU_MR = 0x00000800;			// normal mode, no parity
  *AT91C_DBGU_BRGR = 26;			// MCK/16/26 = 115200 bps
  *AT91C_DBGU_CR = 0x00000150;			// TX, RX enable

  while(1)
    { for(i=1, x=0.001; i <= 200; i++, x += 0.001) // loop by 200 times
        { printf("printf() function test for DBGU : %3d  %5.3f\n", i, x);
          Delay_ms(500);
        }
      Beep();
    }
}
