/* ========================================================================== */
/*	    Xtest18_6.c : User-defined printf() Function for USART0	      */
/* ========================================================================== */
/*			  Designed and programmed by Duck-Yong Yoon in 2007.  */

#include <stdio.h>
#include "AT91SAM7S256.h"
#include "lib_AT91SAM7S256.h"
#include "OK7S256ads.h"

void USART0_TX_char(unsigned int data)		/* transmit a character by USART0 */
{
  while(!(*AT91C_US0_CSR & 0x0002));		// wait until TXRDY=1

  *AT91C_US0_THR = data;			// transmit a character
}

int fputc(int ch, FILE *f)			// for the user-defined printf()
{
  if(ch == '\n')
    { USART0_TX_char('\r');			// (CR character = 0x0D)
      USART0_TX_char('\n');			// (LF character = 0x0A)
    }
  else
    USART0_TX_char(ch);

  return ch;
}

int main(void)
{ unsigned int i;
  double x;

  MCU_initialize();				// initialize AT91SAM7S256 & kit
  Delay_ms(50);					// wait for system stabilization
  LCD_initialize();				// initialize text LCD
  Beep();

  LCD_string(0x80,"    printf()    ");		// display title
  LCD_string(0xC0,"   for USART0   ");

  AT91F_US0_CfgPMC();				// enable clock of USART0
  AT91F_PIO_CfgPeriph(AT91C_BASE_PIOA,AT91C_PA6_TXD0|AT91C_PA5_RXD0,0);
						// TXD0, RXD0 by Peripheral A

  *AT91C_US0_MR = 0x100008C0;			// normal mode, no parity
						// 8 data, 1 stop, LSB first, 16x
  *AT91C_US0_BRGR = 26;				// MCK/16/26 = 115200 bps
  *AT91C_US0_CR = 0x00000150;			// TX, RX enable

  while(1)
    { for(i=1, x=0.001; i <= 200; i++, x += 0.001) // loop by 200 times
        { printf("printf() function test for USART0 : %3d  %5.3f\n", i, x);
          Delay_ms(500);
        }
      Beep();
    }
}
